@@ -443,15 +443,16 @@ CollectNicknames( NSSCertificate *c, void *data)
 	/* allocate the node */
 	node = (stringNode*)PORT_ArenaAlloc(names->arena, sizeof(stringNode));
 	if ( node == NULL ) {
-	    return(PR_FAILURE);
+	    PORT_Free(nickname);
+	    return PR_FAILURE;
 	}
 
 	/* copy the string */
 	len = PORT_Strlen(nickname) + 1;
 	node->string = (char*)PORT_ArenaAlloc(names->arena, len);
 	if ( node->string == NULL ) {
-	    if (nickname) PORT_Free(nickname);
-	    return(PR_FAILURE);
+	    PORT_Free(nickname);
+	    return PR_FAILURE;
 	}
 	PORT_Memcpy(node->string, nickname, len);
 
@@ -494,7 +495,7 @@ CERT_GetCertNicknames(CERTCertDBHandle *handle, int what, void *wincx)
     names->totallen = 0;
 
     /* make sure we are logged in */
-    (void) pk11_TraverseAllSlots(NULL, NULL, wincx);
+    (void) pk11_TraverseAllSlots(NULL, NULL, PR_TRUE, wincx);
    
     NSSTrustDomain_TraverseCertificates(handle,
 					    CollectNicknames, (void *)names);
@@ -672,12 +673,12 @@ CERT_DistNamesFromNicknames(CERTCertDBHandle *handle, char **nicknames,
     
     arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
     if (arena == NULL) goto loser;
-    dnames = (CERTDistNames*)PORT_Alloc(sizeof(CERTDistNames));
+    dnames = PORT_ArenaZNew(arena, CERTDistNames);
     if (dnames == NULL) goto loser;
 
     dnames->arena = arena;
     dnames->nnames = nnames;
-    dnames->names = names = (SECItem*)PORT_Alloc(nnames * sizeof(SECItem));
+    dnames->names = names = PORT_ArenaZNewArray(arena, SECItem, nnames);
     if (names == NULL) goto loser;
     
     for (i = 0; i < nnames; i++) {
@@ -1046,8 +1047,8 @@ loser:
     nssUsage.anyUsage = PR_FALSE;
     nssUsage.nss3usage = usage;
     nssUsage.nss3lookingForCA = PR_FALSE;
-    stanChain = NSSCertificate_BuildChain(stanCert, NULL, &nssUsage, NULL,
-					  NULL, 0, NULL, NULL, td, cc);
+    stanChain = NSSCertificate_BuildChain(stanCert, NULL, &nssUsage, NULL, NULL,
+					  CERT_MAX_CERT_CHAIN, NULL, NULL, td, cc);
     if (!stanChain) {
 	PORT_SetError(SEC_ERROR_UNKNOWN_ISSUER);
 	return NULL;
