@@ -35,7 +35,7 @@
  * ***** END LICENSE BLOCK ***** */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: devmod.c,v $ $Revision: 1.7 $ $Date: 2005/01/20 02:25:47 $";
+static const char CVS_ID[] = "@(#) $RCSfile: devmod.c,v $ $Revision: 1.7.28.2 $ $Date: 2007/10/10 09:12:01 $";
 #endif /* DEBUG */
 
 #ifndef NSSCKEPV_H
@@ -303,9 +303,13 @@ nssModule_Unload (
 )
 {
     PRStatus nssrv = PR_SUCCESS;
+    char *disableUnload = NULL;
     if (mod->library) {
 	(void)CKAPI(mod->epv)->C_Finalize(NULL);
-	nssrv = PR_UnloadLibrary(mod->library);
+	disableUnload = PR_GetEnv("NSS_DISABLE_UNLOAD");
+	if (!disableUnload) {
+	    nssrv = PR_UnloadLibrary(mod->library);
+	}
     }
     /* Free the slots, yes? */
     mod->library = NULL;
