@@ -79,6 +79,7 @@ memleak_init()
 	OLD_LIBRARY_PATH=${LD_LIBRARY_PATH}
 	TMP_LIBDIR="${HOSTDIR}/tmp$$"
 	TMP_STACKS="${HOSTDIR}/stacks$$"
+	TMP_COUNT="${HOSTDIR}/count$$"
 	
 	PORT=${PORT:-8443}
 	
@@ -98,7 +99,7 @@ memleak_init()
 	IGNORED_STACKS="${QADIR}/memleak/ignored"
 	
 	gline=`echo ${OBJDIR} | grep "_64_"`
-	if [ "${gline}" ] ; then
+	if [ -n "${gline}" ] ; then
 		BIT_NAME="64"
 	else
 		BIT_NAME="32"
@@ -107,6 +108,7 @@ memleak_init()
 	case "${OS_NAME}" in
 	"SunOS")
 		DBX=`which dbx`
+		AWK=nawk
 		
 		if [ $? -eq 0 ] ; then
 			echo "${SCRIPTNAME}: DBX found: ${DBX}"
@@ -141,6 +143,7 @@ memleak_init()
 		;;
 	"Linux")
 		VALGRIND=`which valgrind`
+		AWK=awk
 		
 		if [ $? -eq 0 ] ; then
 			echo "${SCRIPTNAME}: Valgrind found: ${VALGRIND}"
@@ -167,11 +170,21 @@ memleak_init()
 		;;
 	esac
 
+	if [ "${BUILD_OPT}" -eq 1 ] ; then
+		OPT="OPT"
+	else 
+		OPT="DBG"
+	fi
+
+	NSS_DISABLE_UNLOAD="1"
+	export NSS_DISABLE_UNLOAD
+
 	SELFSERV_ATTR="-D -p ${PORT} -d ${SERVER_DB} -n ${HOSTADDR} -e ${HOSTADDR}-ec -w nss -c ABCDEF:C001:C002:C003:C004:C005:C006:C007:C008:C009:C00A:C00B:C00C:C00D:C00E:C00F:C010:C011:C012:C013:C014cdefgijklmnvyz -t 5"
 	TSTCLNT_ATTR="-p ${PORT} -h ${HOSTADDR} -c j -f -d ${CLIENT_DB} -w nss"
 
 	tbytes=0
 	tblocks=0
+	truns=0
 }
 
 ########################### memleak_cleanup ############################
@@ -179,6 +192,8 @@ memleak_init()
 ########################################################################
 memleak_cleanup()
 {
+	unset NSS_DISABLE_UNLOAD
+	
 	. ${QADIR}/common/cleanup.sh
 }
 
@@ -291,7 +306,8 @@ run_selfserv_dbx()
 	cat << EOF_DBX > ${DBX_CMD}
 dbxenv follow_fork_mode parent
 dbxenv rtc_mel_at_exit verbose
-check -leaks -match 16 -frames 16
+dbxenv rtc_biu_at_exit verbose
+check -memuse -match 16 -frames 16
 run ${SERVER_OPTION} ${SELFSERV_ATTR}
 EOF_DBX
 	
@@ -314,8 +330,8 @@ run_selfserv_valgrind()
 	echo "PATH=${PATH}"
 	echo "LD_LIBRARY_PATH=${LD_LIBRARY_PATH}"
 	echo "${SCRIPTNAME}: -------- Running selfserv under Valgrind:"
-	echo "${VALGRIND} --tool=memcheck --leak-check=yes --show-reachable=yes --num-callers=50 selfserv ${SELFSERV_ATTR}"
-	${VALGRIND} --tool=memcheck --leak-check=yes --show-reachable=yes --num-callers=50 selfserv ${SELFSERV_ATTR}
+	echo "${VALGRIND} --tool=memcheck --leak-check=yes --show-reachable=yes --partial-loads-ok=yes --leak-resolution=high --num-callers=50 selfserv ${SELFSERV_ATTR}"
+	${VALGRIND} --tool=memcheck --leak-check=yes --show-reachable=yes --partial-loads-ok=yes --leak-resolution=high --num-callers=50 selfserv ${SELFSERV_ATTR}
 }
 
 ############################ strsclnt_attr #############################
@@ -360,7 +376,8 @@ run_strsclnt_dbx()
 		cat << EOF_DBX > ${DBX_CMD}
 dbxenv follow_fork_mode parent
 dbxenv rtc_mel_at_exit verbose
-check -leaks -match 16 -frames 16
+dbxenv rtc_biu_at_exit verbose
+check -memuse -match 16 -frames 16
 run ${CLIENT_OPTION} ${STRSCLNT_ATTR}
 EOF_DBX
 
@@ -390,8 +407,8 @@ run_strsclnt_valgrind()
 	for cipher in ${cipher_list}; do
 		strsclnt_attr ${cipher}
 		echo "${SCRIPTNAME}: -------- Trying cipher ${cipher} under Valgrind:"
-		echo "${VALGRIND} --tool=memcheck --leak-check=yes --show-reachable=yes --num-callers=50 strsclnt ${STRSCLNT_ATTR}"
-		${VALGRIND} --tool=memcheck --leak-check=yes --show-reachable=yes --num-callers=50 strsclnt ${STRSCLNT_ATTR}
+		echo "${VALGRIND} --tool=memcheck --leak-check=yes --show-reachable=yes --partial-loads-ok=yes --leak-resolution=high --num-callers=50 strsclnt ${STRSCLNT_ATTR}"
+		${VALGRIND} --tool=memcheck --leak-check=yes --show-reachable=yes --partial-loads-ok=yes --leak-resolution=high --num-callers=50 strsclnt ${STRSCLNT_ATTR}
 	done
 
 	echo "PATH=${PATH}"
@@ -485,39 +502,75 @@ run_ciphers_client()
 ########################################################################
 parse_logfile_dbx()
 {
-	in_mel=0
+	${AWK} '
+	BEGIN {
+		in_mel = 0
+		mel_line = 0
+		bytes = 0
+		blocks = 0
+		runs = 0
+		stack_string = ""
+		bin_name = ""
+	}
+	/Memory Leak \(mel\):/ ||
+	/Possible memory leak -- address in block \(aib\):/ ||
+	/Block in use \(biu\):/ {
+		in_mel = 1
+		stack_string=""
+		next
+	}
+	in_mel == 1 && /^$/ {
+		print bin_name stack_string
+		in_mel = 0
+		mel_line = 0
+		next
+	}
+	in_mel == 1 {
+		mel_line += 1;
+	}
+	/Found leaked block of size/ {
+		bytes += $6
+		blocks += 1
+		next
+	}
+	/Found .* leaked blocks/ {
+		bytes += $8
+		blocks += $2
+		next
+	}
+	/Found block of size/ {
+		bytes += $5
+		blocks += 1
+		next
+	}
+	/Found .* blocks totaling/ {
+		bytes += $5
+		blocks += $2
+		next
+	}
+	mel_line > 2 {
+		gsub(/\(\)/,"")
+		new_line = $2;
+		stack_string = "/" new_line stack_string
+		next
+	}
+	/^Running: / {
+		bin_name = $2
+		next
+	}
+	/execution completed/ {
+		runs += 1
+		next
+	}
+	END {
+		print "# " bytes " bytes " blocks " blocks in " runs " runs" > "/dev/stderr";
+	}' 2> ${TMP_COUNT}
 	
-	while read line
-	do
-		if [ "${line}" = "Memory Leak (mel):" -o "${line}" = "Possible memory leak -- address in block (aib):" ] ; then
-			in_mel=1
-			mel_line=0
-			stack_string=""
-		fi
-		
-		if [ -z "${line}" ] ; then
-			if [ ${in_mel} -eq "1" ] ; then
-				in_mel=0
-				echo "${stack_string}"
-			fi
-		fi
-			
-		if [ ${in_mel} -eq 1 ] ; then
-			mel_line=`expr ${mel_line} + 1`
-			
-			if [ ${mel_line} -ge 4 ] ; then
-				new_line=`echo "${line}" | cut -d" " -f2 | cut -d"(" -f1`
-				stack_string="/${new_line}${stack_string}"
-			fi
-				
-			gline=`echo "${line}" | grep "Found leaked block of size"`
-			if [ "${gline}" ] ; then
-				lbytes=`echo "${line}" | sed "s/Found leaked block of size \(.*\) bytes.*/\1/"`
-				tbytes=`expr "${tbytes}" + "${lbytes}"`
-				tblocks=`expr "${tblocks}" + 1`
-			fi
-		fi	
-	done
+	read hash lbytes bytes_str lblocks blocks_str in_str lruns rest < ${TMP_COUNT}
+	tbytes=`expr "${tbytes}" + "${lbytes}"`
+	tblocks=`expr "${tblocks}" + "${lblocks}"`
+	truns=`expr "${truns}" + "${lruns}"`
+	rm ${TMP_COUNT}
 }
 
 ######################## parse_logfile_valgrind ########################
@@ -525,133 +578,69 @@ parse_logfile_dbx()
 ########################################################################
 parse_logfile_valgrind()
 {
-	in_mel=0
-	in_sum=0
+	${AWK} '
+	 BEGIN {
+		 in_mel=0;
+		 in_sum=0;
+		 bytes=0;
+		 blocks=0;
+		 runs=0;
+		 stack_string="";
+		 bin_name=""; }
+	 !/==[0-9]*==/ { 
+		if ( $1 == "'${VALGRIND}'" ) 
+			bin_name = $8 ; 
+		next;
+	 }
+	 /blocks are/ {
+		in_mel = 1;
+		stack_string="";
+		next;
+	 }
+	 /LEAK SUMMARY/ {
+		in_sum=1;
+		runs += 1;
+		next;
+	 }
+	 /^==[0-9]*== *$/ { 
+	    	if (in_mel)
+			print bin_name stack_string;
+		in_sum = 0;
+		in_mel = 0;
+		next;
+	 }
+	 in_mel == 1 {	
+		new_line = $4;
+		if ( new_line == "(within")
+			new_line = "*";
+		stack_string = "/" new_line stack_string;
+	 }
+	 in_sum == 1 {
+		for (i=2; i <= NF; i++) {
+			if ($i == "bytes") {
+				str = $(i-1);
+				gsub(",","",str);
+				bytes += str;
+			}
+			if ($i == "blocks.") {
+				str = $(i-1);
+				gsub(",","",str);
+				blocks += str;
+			}
+		}
+	 }
+	 END {
+		print "# " bytes " bytes " blocks " blocks in " runs " runs" > "/dev/stderr";
+	 }' 2> ${TMP_COUNT}
 
-	while read line
-	do
-		gline=`echo "${line}" | grep "^=="`
-		if [ ! "${gline}" ] ; then
-			continue
-		fi
-
-		line=`echo "${line}" | sed "s/==[0-9]*==\s*\(.*\)/\1/"`
-
-		gline=`echo "${line}" | grep "blocks are"`
-		if [ "${gline}" ] ; then
-			in_mel=1
-			mel_line=0
-			stack_string=""
-		else
-			gline=`echo "${line}" | grep "LEAK SUMMARY"`
-			if [ "${gline}" ] ; then
-				in_sum=1
-				mel_line=0
-			fi
-		fi
-		
-		if [ -z "${line}" ] ; then
-			if [ ${in_mel} -eq 1 ] ; then
-				in_mel=0
- 				echo "${stack_string}"
-			elif [ ${in_sum} -eq 1 ] ; then
-				in_sum=0
-			fi
-		fi
-			
-		if [ ${in_mel} -eq 1 ] ; then
-			mel_line=`expr ${mel_line} + 1`
-			
-			if [ ${mel_line} -ge 2 ] ; then
-				new_line=`echo "${line}" | sed "s/[^:]*:\ \(\S*\).*/\1/"`
-				if [ "${new_line}" = "(within" ] ; then
-					new_line="*"
-				fi
-				stack_string="/${new_line}${stack_string}"
-			fi
-		elif [ ${in_sum} -eq 1 ] ; then
-			mel_line=`expr ${mel_line} + 1`
-			
-			if [ ${mel_line} -ge 2 ] ; then
-				gline=`echo "${line}" | grep "bytes.*blocks"`
-				if [ "${gline}" ] ; then
-					lbytes=`echo "${line}" | sed "s/.*: \(.*\) bytes.*/\1/" | sed "s/,//g"`
-					lblocks=`echo "${line}" | sed "s/.*bytes in \(.*\) blocks.*/\1/" | sed "s/,//g"`
-
-					tbytes=`expr "${tbytes}" + "${lbytes}"`
-					tblocks=`expr "${tblocks}" + "${lblocks}"`
-				else
-					in_sum=0
-				fi
-			fi
-		fi		
-	done
-}
-
-############################# log_compare ##############################
-# local shell function to check if selected stack is found in the list
-# of ignored stacks
-########################################################################
-log_compare()
-{
-	BUG_ID=""
-
-	while read line
-	do
-		LINE="${line}"
-		STACK="${stack}"
-
-		if [ "${LINE}" = "${STACK}" ] ; then
-			return 0
-		fi
-		
-		NEXT=0
-
-		gline=`echo "${LINE}" | grep '^#'`
-		if [ "${gline}" ] ; then
-			BUG_ID="${LINE}"
-			NEXT=1
-		fi
-		
-		gline=`echo "${LINE}" | grep '*'`
-		if [ ! "${gline}" ] ; then
-			NEXT=1
-		fi
-		
-		while [ "${LINE}" != "" -a ${NEXT} -ne 1 ]
-		do
-			L_WORD=`echo "${LINE}" | cut -d '/' -f1`
-			gline=`echo "${LINE}" | grep '/'`
-			if [ "${gline}" ] ; then
-				LINE=`echo "${LINE}" | cut -d '/' -f2-`
-			else
-				LINE=""
-			fi
-
-			S_WORD=`echo "${STACK}" | cut -d '/' -f1`
-			gline=`echo "${STACK}" | grep '/'`
-			if [ "${gline}" ] ; then
-				STACK=`echo "${STACK}" | cut -d '/' -f2-`
-			else
-				STACK=""
-			fi
-			
-			if [ "${L_WORD}" = "**" ] ; then
-				return 0
-			fi
-			
-			if [ "${L_WORD}" != "${S_WORD}" -a "${L_WORD}" != "*" -a "${S_WORD}" != "*" ] ; then
-				NEXT=1
-				break
-			fi
-		done
-
-		if [ "${LINE}" = "" -a "${STACK}" = "" ] ; then
-			return 0
-		fi
-	done
-
-	return 1
+	# sigh it would be nice to just pipe stderr and let stdout go by I've never been 
+	# able to convince any shell to do that correctly, so we are reduced to using a temp
+	# file
+	read hash lbytes bytes_str lblocks blocks_str in_str lruns rest < ${TMP_COUNT}
+	tbytes=`expr "${tbytes}" + "${lbytes}"`
+	tblocks=`expr "${tblocks}" + "${lblocks}"`
+	truns=`expr "${truns}" + "${lruns}"`
+	rm ${TMP_COUNT}
 }
 
 ############################# check_ignored ############################
@@ -659,24 +648,65 @@ log_compare()
 ########################################################################
 check_ignored()
 {
-	ret=0
-
-	while read stack
-	do
-		log_compare < ${IGNORED_STACKS}
-		if [ $? -eq 0 ] ; then
-			if [ ${BUG_ID} != "" ] ; then
-				echo "IGNORED STACK (${BUG_ID}): ${stack}"
-			else
-				echo "IGNORED STACK: ${stack}"
-			fi
-		else
-			ret=1
-			echo "NEW STACK: ${stack}"
-		fi
-	done
-
-	return ${ret}
+    ${AWK} -F/ '
+	BEGIN {
+		ignore = "'${IGNORED_STACKS}'"
+		# read in the ignore file
+		BUGNUM = ""
+		count = 0
+		new = 0
+		while ((getline line < ignore) > 0)  {
+			if (line ~ "^#[0-9]+") {
+				BUGNUM = line
+			} else if (line ~ "^#") {
+				continue
+			} else if (line == "") {
+				continue
+			} else {
+				bugnum_array[count] = BUGNUM
+ 				# Create a regular expression for the ignored stack:
+ 				# replace * with % so we can later replace them with regular expressions
+ 				# without messing up everything (the regular expressions contain *)
+ 				gsub("\\*","%",line)
+ 				# replace %% with .*
+ 				gsub("%%",".*",line)
+ 				# replace % with [^/]*
+ 				gsub("%","[^/]*",line)
+ 				# add ^ at the beginning
+ 				# add $ at the end
+ 				line_array[count] = "^" line "$"
+				count++
+			}
+		}
+	}
+	{
+		match_found=0
+		# Look for matching ignored stack
+	 	for (i=0 ; i < count; i++) {
+			if ($0 ~ line_array[i]) {
+				# found a match
+				match_found = 1
+				bug_found = bugnum_array[i]
+				break
+			}
+		}
+		# Process result
+		if (match_found == 1 ) {
+				if (bug_found != "") {
+					print "IGNORED STACK (" bug_found "): " $0
+				} else {
+					print "IGNORED STACK: " $0
+				}
+		} else {
+				print "NEW STACK: " $0
+				new = 1
+		}
+	}
+	END {
+		exit new
+	}'
+	ret=$?
+	return $ret
 }
 
 ############################### parse_log ##############################
@@ -700,8 +730,9 @@ log_parse()
 cnt_total()
 {
 	echo ""
-	echo "TinderboxPrint:Lk bytes: ${tbytes}"
-	echo "TinderboxPrint:Lk blocks: ${tblocks}"
+	echo "TinderboxPrint:${OPT} Lk bytes: ${tbytes}"
+	echo "TinderboxPrint:${OPT} Lk blocks: ${tblocks}"
+	echo "TinderboxPrint:${OPT} # of runs: ${truns}"
 	echo ""
 }
 
